"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClusterManager = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const cluster_constants_1 = require("./cluster.constants");
const utils_1 = require("../utils");
const errors_1 = require("../errors");
/**
 * Manager for cluster clients.
 *
 * @public
 */
let ClusterManager = class ClusterManager {
    constructor(clusterClients) {
        this.clusterClients = clusterClients;
    }
    /**
     * Retrieves all cluster clients.
     */
    get clients() {
        return this.clusterClients;
    }
    /**
     * Retrieves a cluster client by namespace.
     */
    getClient(namespace = cluster_constants_1.DEFAULT_CLUSTER_NAMESPACE) {
        const client = this.clusterClients.get(namespace);
        if (!client)
            throw new errors_1.ClientNotFoundError((0, utils_1.parseNamespace)(namespace), 'cluster');
        return client;
    }
};
ClusterManager = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(cluster_constants_1.CLUSTER_CLIENTS)),
    tslib_1.__metadata("design:paramtypes", [Object])
], ClusterManager);
exports.ClusterManager = ClusterManager;
