"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisManager = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const redis_constants_1 = require("./redis.constants");
const utils_1 = require("../utils");
const errors_1 = require("../errors");
/**
 * Manager for redis clients.
 *
 * @public
 */
let RedisManager = class RedisManager {
    constructor(redisClients) {
        this.redisClients = redisClients;
    }
    /**
     * Retrieves all redis clients.
     */
    get clients() {
        return this.redisClients;
    }
    /**
     * Retrieves a redis client by namespace.
     */
    getClient(namespace = redis_constants_1.DEFAULT_REDIS_NAMESPACE) {
        const client = this.redisClients.get(namespace);
        if (!client)
            throw new errors_1.ClientNotFoundError((0, utils_1.parseNamespace)(namespace), 'redis');
        return client;
    }
};
RedisManager = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(redis_constants_1.REDIS_CLIENTS)),
    tslib_1.__metadata("design:paramtypes", [Object])
], RedisManager);
exports.RedisManager = RedisManager;
